comment *

    This program is a MachZ BUR Extension which will execute out of the MachZ
    on-chip SRAM under control of the BUR.  The program is a flash programmer
    for AMD 8-bit chips This programmer reads a variable-length payload from
    the Z-tag input port and writes the data into the flash device.  The
    programmer requires one parameter record which is the start address within
    the flash.  This programmer will write the payload to flash chip, starting
    from the address defined in the parameter.

    There is no checking for overruns.  If the chip is smaller than needed,
    wrap-around will occur.

    (c)2000 ZF Linux Devices, Inc.


    revision history
    ----------------
    CRC 12-26-00        changed structure, removed macros, added comments


  Target Chip:          AMD AM29F0xx in MachZ Integrated Development System
  Size:                 2 Megabyte
  Chipselect:           ms_cs0
  Mode:                 8-bit
  Chip Address:         defined by Parameter record (use 0x000C0000)
  Window size:          10000H (64K)

    This code executes as BUR "Load and execute" function.  It will fetch
    parameter record and payload code following to the executable code in
    dongle.  Compile using MASM 6.11 with following command line:

        ml /Fl /Zm /Fm AMDFLASH.ASM

    The resulting .COM file can be directly used as payload for BUR "Upload and
    Execute" function

  endcomment *

.486p


ZFINDEX         EQU     218H    ; ZF-logic Index
ZFRW8           EQU     219H
ZFRW1632        EQU     21AH


MEMCS           EQU     0                       ; use chip select 0
PRGBASE         EQU     0E0000h                 ; where to create memory window
CS_PAGE         EQU     ((26h+(MEMCS*12))+8)    ; ZF Logic Page Register for MEMCS
CS_BASE         EQU     (26h+(MEMCS*12))        ; base register for specified chipselect
CS_SIZE         EQU     ((26h+(MEMCS*12))+4)    ; ZF Logic Size Register for MEMCS
MEMCONTROLL     EQU     5AH                     ; ZF Logic Memory Control Low
ZT_WRITE        EQU     5EH                     ; ZF Logic ZT Data Write Register


_16BIT_RW       EQU     00000000b
_8BIT_RW        EQU     00000001b
WINDOWMODE      EQU     _8BIT_RW                ; "global" parameter

CR              EQU     0DH
LF              EQU     0AH
TRUE            EQU     1
FALSE           EQU     0

                assume cs:code; es:code; ds:nothing
code            segment USE16 public
                org     0

START:
                push    cs
                pop     es
                call    PRINT_AREGS     ; demonstration uses debugsub.asm
                jmp     start1

                include debugsub.asm    ; not needed


message1        db      CR,LF, 'MachZ Integrated Development System AMD Demo BUR Flash Programmer',CR,LF,0
message2        db             '-----------------------------------------------------------------',CR,LF,0
message3        db      '0x',0
message4        db      ': Erasing .. ',0
message5        db      'Flash programming start address missing!',CR,LF,0
message6        db      'Programming starts from 0x',0
message7        db      'Device: Mfg=',0
message8        db      ' DevID=',0
message9        db      'Detected Flash Device: ',0
message10       db      'Programming .. ',0
message11       db      'RLE basket checksum error ',0
message12       db      'Data CRC failure: ',0
message13       db      'Chip Not Supported',CR,LF,0
message14       db      'FAILED!',CR,LF,0
message15       db      'OK!',CR,LF,0
message16       db      'Data CRC was OK!',CR,LF,0
message17       db      'Programming Failure ...',CR,LF,0

fIDError        db      FALSE
FlashBase       dd      ?    ; variable to store starting address in chip

;   ----------------------------------------------------------------
;   subroutine print - print ASCIIZ string. ES:DI -> string on entry
;   ----------------------------------------------------------------

print   proc
        pusha
        xor     cx, cx
        call    SerSend
        popa
        ret
print   endp



start1:
        LEA     DI, message1    ; 'MachZ Integrated Development System AMD Demo BUR Flash Programmer'
        call    print
        LEA     DI, message2    ; ------------------------------------------------------------------------',CR,LF,0
        call    print

        call    ZTPrepareRead   ; BUR function use before ZTRead

        call    ParmRecord2EAX  ; fetch parameter record to EAX
        jnc     ParmOK          ; procedure in flashpgm.asm

    comment * in this generalized program, you can specify the target address
    within the flash chip.  If you are "burning" in the Phoenix BIOS in the
    IDS, using the 2 MB IDS AMD Flash Chip, you use starting address 0x1C0000.
    We could hard code this into the program, but leave it here as an
    instructional example.  *


        LEA     DI, message5    ; 'Flash programming start address missing!'
        call    print
        jmp     ExitPgmrFail
ParmOK:
        and     eax, 0FFFFF000h ;  ZF Logic BASE reg: rightmost = 0
        mov     FlashBase, eax
        push    eax             ; save for a moment

        LEA     DI, message6    ; 'Programming starts from 0x'
        call    print

        pop     eax             ; restore eax
        call    SerOut32        ; BUR API proc writes EAX to Serial
        call    CRLF            ; BUR API proc writes CRLF to Serial

        ; Get the Basket Command

        MOV     BX, OFFSET ExitPgmrFail ; pass in fail address
        Call    process_basket_header   ; our procedure in flashsub.asm

        ; Create Memory window for accessing the flash device

        Call    Create_Memory_Window

        ; Time to program some flash.

    comment * Check, whenever or not we have supported part on-board.  A good
    reference here is the AMD data sheet for the Am29F016D, the publication
    21444 Rev E on the AMD web site. The chip used on the MachZ Integrated
    Development system has a Manufacturer ID of 01 and a device ID of 0xAD. *


        mov     al, 90h                 ; send out the 1-2-3rd bus
        call    JEDEC_Cmd_8             ;  cycles as per data sheet

        push    (PRGBASE/10h)
        pop     ds
        xor     si, si                  ; read from 0-1 and get
        mov     ax, ds:[si]             ;  Manu and Device ID
        mov     bx, ax                  ; save

        MOV     DI, OFFSET Message7     ; 'Device: Mfg='
        call    print
        call    SerOut8


;       if (MfgID != 1) fID_error = TRUE;

        CMP     AL, 1                   ; hardwired for AMD
        JE      endMANUtest
        MOV     fIDError, TRUE
endMANUtest:

        MOV     DI, OFFSET Message8     ; ' DevID='
        call    print
        shr     ax, 8

;       if (DeviceID != 0xAD) fID_error = TRUE;

        CMP     AL, 0ADH                ; hardwired for AMD 29F016D
        JE      endIDtest
        MOV     fIDError, TRUE
endIDtest:

        call    SerOut8
        call    CRLF

        mov     al, 0F0h                ; reset command to
        mov     ds:[si], al             ; end identification mode


;       if (fIDerror)

        CMP     fIDerror, TRUE          ; hardwired for AMD 29F016D
        JNE     ChipSupported
        LEA     DI, Message13           ; chip not supported
        CALL    Print
        JMP     ExitPgmrFail


ChipSupported:

        ; ====== Basket handling begins here ======

    COMMENT * Note, that in order to maintain correct checksum for basket, we
    must calculate data CRC for RLE header as well!  Thus we resetCRC prior to
    calling process_RLE_header but do not verify the checksum until after all
    the basket data is read.  *

        call    ResetCRC                ; BUR API routine prior to INT 17H
        MOV     AX, OFFSET ExitPgmrFail
        call    process_RLE_header      ; process RLE header if any
        mov     ebp, BasketSize         ; total bytes to program

PrgLoop:                                ; Erase sector now
        LEA     DI, message3            ; '0x'
        CALL    Print
        mov     eax, FlashBase
        call    SerOut32                ; write AX as xxxx in hex

        LEA     DI, message4            ; ': Erasing .. '
        CALL    Print

 ;  There are two 3-byte sequences to do a sector erase.

        push    (PRGBASE/10h)
        pop     fs              ; fs:0 points to our memory window
        xor     si, si

        mov     al, 80h         ; sector erase command part 1
        call    JEDEC_Cmd_8
        mov     al, 30h         ; sector erase command part 2
        call    JEDEC_Cmd_8

    comment * Erase is now in progress.  Check, when we are ready.  There is a
    bur variable which is incremented 18.2 times/second.  182 times is a 10
    second timeout.  *

        call    DSBX2Var                ; DS:BX -> BUR data area (RAM)
        mov     ax, 182
        mov     ds:[bx.CountDown], ax   ; gives 10sec. timeout

        mov     dx, ZFINDEX
        mov     al, ZT_WRITE            ; ZT_SIG_OUT
        out     dx, al                  ;
        inc     dx                      ;
        in      al, dx                  ;
        and     al, 11110101b           ; turn off LED's
        out     dx, al

        call    ZTPrepareRead           ; ZFLogic back to track

    comment * to test for erase, you only need to look at D7 which is forced to
    a 0 until the erase is done. The FFFFFFF test below will accomplish the
    same thing. *


@@:
        cmp     word ptr ds:[bx.CountDown], 0   ; did we timeout
        jz      @f
        cmp     dword ptr fs:[si], 0FFFFFFFFh   ; D7 = 1 is a
        jnz     @b                              ; loop until erase completes
@@:
        mov     word ptr ds:[bx.CountDown], 0   ; reset timer, so we will not loose our blinking LED's
        cmp     dword ptr fs:[si], 0FFFFFFFFh   ; test again for branch
        jz      @f

        LEA     DI, message14   ; 'FAILED!'
        call    print

        jmp     ExitPgmrFail
@@:

        ; All set. Programming ...

        LEA     DI, message10           ; 'Programming .. '
        call    print

        mov     ecx, 64*1024
        cmp     ecx, ebp                ; EBP is # of bytes left to program
        jbe     @f
        mov     ecx, ebp                ; if less than 64K left, program it
@@:
        cmp     ecx, 0
        jz      PgmDone                 ; don't do anything if basket size is 0

        push    (PRGBASE/10h)
        pop     ds
        xor     si, si

BytePrgLoop:
        mov     al, 0A0h                ; "program" command
        call    JEDEC_Cmd_8

        call    ZTRead_RLE              ; read and do INT 17 to maintain checksum

        mov     [si], al                ; write byte
        mov     bl, al                  ; save byte for future compare

        ; on the extreme case we can have PCI backside clock 80Mhz. ISA
        ; divider is 8, so we have 10M ISA bus clock. Programming cycle can be max.
        ; 50us, so we need to wait here about 500 ISA cycles to kill that time.


        mov     di, 500
WaitWriting:
        cmp     ds:[si], bl
        jz      ByteOk
        in      al, 80h                 ; create one ISA cycle for delay
        dec     di
        jnz     WaitWriting

        LEA     DI, Message14   ; 'FAILED!'
        CALL    print
        jmp     ExitPgmrFail

ByteOk:
        inc     si                      ; advance address to next
        loopd   BytePrgLoop
PgmDone:
        LEA     DI, message15   ; 'OK!'
        call    print

        ; Anything left?    if ((ebp-64K)<= 0) goto @f

        sub     ebp, 64*1024
        jz      @f
        jc      @f

    ; move to next page - note that CS0 and page size are fixed, but that
    ; flashbase came in from the parameter record

        add     FlashBase, 64*1024      ; update page to next
        Call    Set_Flash_Offset        ; uses CS0, FlashBase

        jmp     PrgLoop

        ; Now when programming is done, go check the data checksum if it was RLE image

        cmp     BasketType, 0FFh
        jz      @f

        mov     eax, RLE_CheckSum
        cmp     eax, RLE_Chk            ; compare against calculated checksum
        jz      @f

        LEA     DI, message11   ; 'RLE basket checksum error '
        call    print

        call    SerOut32
        mov     al, ' '
        call    SerSend2
        mov     eax, RLE_Chk
        call    SerOut32
        call    CRLF

        jmp     ExitPgmrFail
@@:
        call    ZTPrepareRead

        ; Get original checksum

        call    ZTRead
        shl     ax, 8
        call    ZTRead
        xchg    al, ah

        call    DSBX2Var                ; get variables block to DS:BX
        cmp     word ptr ds:[bx.YModemCRChi_C], ax
        jz      CRCOK

        LEA     DI, message12   ; 'Data CRC failure: '
        call    print

        call    SerOut16
        mov     al, ' '
        call    SerSend2
        mov     ax, word ptr ds:[bx.YModemCRChi_C]
        call    SerOut16
        call    CRLF
        jmp     ExitPgmrFail
CRCOK:

        LEA     DI, message16   ; 'Data CRC was OK!'
        CALL    print
        jmp     ExitPgmrOk

ExitPgmrFail: ; Light up RED LED and do not do anything else!

        mov     dx, ZFINDEX
        mov     al, ZT_WRITE            ;ZT_SIG_OUT
        out     dx, al
        inc     dx
        in      al, dx
        and     al, 11110101b
        or      al, ZT_LED_RED          ;00001000b
        out     dx, al

        LEA     DI, message17   ; 'Programming Failure ...'
        CALL    print
        jmp     $

ExitPgmrOk:
        ; Light up GREEN LED and continue with BUR
        mov     dx, ZFINDEX
        mov     al, ZT_WRITE            ;ZT_SIG_OUT - ZTAGWRITE
        out     dx, al
        inc     dx
        in      al, dx
        and     al, 11110101b
        or      al, ZT_LED_GREEN        ;00000010b
        out     dx, al
ExitPgmr:
        call    CRLF

        ; Set timer to maximum value. This is useful to prevent the
        ; BUR from blinking with LED's when loading next commands.
        ; This way we maintain our RED or GREEN LED setting

        call    DSBX2Var
        mov     word ptr ds:[bx.CountDown], 0FFFFh

        ; Always exit with ZFL registers prepared for accelerated read!

        call    ZTPrepareRead
        retf                            ; resume with BUR


    ; in-line includes here so no "space" wasted due to segment combination


        include BURAPI.ASM      ; interface to the BUR
        include RLE_MEMW.ASM    ; common routines
        include FLASHPGM.ASM    ; flash programmer common routines
code    ends

END START
ENDS

